#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Part of the Human Connectome - Lifespan Project Task fMRI Battery
Psychopy implementation by Erik K Kastman, Constanza M Vidal Bustamente, and Leah H Somerville
**********************************************************************************************
Tested with Psychopy 1.83.04; Psyexp Experiment Builder XML available upon request
  Peirce, JW (2007) PsychoPy - Psychophysics software in Python. Journal of Neuroscience Methods, 162(1-2), 8-13.
  Peirce, JW (2009) Generating stimuli for neuroscience using PsychoPy. Frontiers in Neuroinformatics, 2:10. doi: 10.3389/neuro.11.010.2008
"""

from __future__ import division  # so that 1/3=0.333 instead of 1/3=0
import datetime
from psychopy import locale_setup, visual, core, data, event, logging, sound, gui
from psychopy.constants import *  # things like STARTED, FINISHED
import numpy as np  # whole numpy lib is available, prepend 'np.'
from numpy import sin, cos, tan, log, log10, pi, average, sqrt, std, deg2rad, rad2deg, linspace, asarray
from numpy.random import random, randint, normal, shuffle
import os  # handy system and path functions
import sys # to get file system encoding

# Common HCP Tools
import hcpcommon
hcpcommon.checkScreenCount()
# Ensure that relative paths start from the same directory as this script
_thisDir = os.path.dirname(os.path.abspath(__file__)).decode(sys.getfilesystemencoding())
os.chdir(_thisDir)

# Store info about the experiment session
expName = 'VISMOTOR'  # from the Builder filename that created this script
expInfo = {u'runNumber': u'1', u'sessionID': u'', u'mriMode': u'scan'}
dlg = gui.DlgFromDict(dictionary=expInfo, title=expName)
if dlg.OK == False: core.quit()  # user pressed cancel
expInfo['date'] = datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')  # add a simple timestamp
expInfo['expName'] = expName

# Data file name stem = absolute path + name; later add .psyexp, .csv, .log, etc
filename = _thisDir + os.sep + u'data/' + '_'.join([expName, expInfo['sessionID'], 'run%s' % expInfo['runNumber'], expInfo['date']])

# An ExperimentHandler isn't essential but helps with data saving
thisExp = data.ExperimentHandler(name=expName, version='',
    extraInfo=expInfo, runtimeInfo=None,
    originPath=None,
    savePickle=True, saveWideText=False,
    dataFileName=filename)
#save a log file for detail verbose info
logFile = logging.LogFile(filename+'.log', level=logging.EXP)
logging.console.setLevel(logging.WARNING)  # this outputs to the screen, not a file

endExpNow = False  # flag for 'escape' or other condition => quit the exp

# Start Code - component code to be run before the window creation
## EKK Screen Edit of compiled script
import pyglet
display = pyglet.window.get_platform().get_default_display()
screens = display.get_screens()
resolution = [screens[-1].width, screens[-1].height]
import yaml
from psychopy import monitors
if not os.path.exists('siteConfig.yaml'): raise IOError('Missing siteConfig.yaml - Please copy configuration text file')
with open('siteConfig.yaml') as f:
    config = yaml.safe_load(f)
mon = monitors.Monitor('newMonitor')
mon.setWidth(config['monitor']['width'])
mon.setDistance(config['monitor']['distance'])
mon.setSizePix(resolution)

# Setup the Window
win = visual.Window(size=resolution, fullscr=False, screen=config['monitor']['screen'], allowGUI=False, allowStencil=False,
    monitor=mon, color=[-1,-1,-1], colorSpace='rgb',
    blendMode='avg', useFBO=True,
    units='deg')
# store frame rate of monitor if we can measure it successfully
expInfo['frameRate']=win.getActualFrameRate()
if expInfo['frameRate']!=None:
    frameDur = 1.0/round(expInfo['frameRate'])
else:
    frameDur = 1.0/60.0 # couldn't get a reliable measure so guess

# Initialize components for Routine "intro"
introClock = core.Clock()
"""
Task Designed and Programmed in Psychtoolbox by 
R. Matthew Hutchinson and Rodrigo Braga

October 2016
Translated to Psychopy by Erik Kastman
"""
import yaml
import itertools

if os.path.exists('siteConfig.yaml'):
    with open('siteConfig.yaml') as f:
        config = yaml.safe_load(f)
else:
    copyMsg = ('Please copy configuration text file '
               '"siteConfig.yaml.example" to "siteConfig.yaml" '
               'and edit it with your trigger and buttons.')
    raise IOError(copyMsg)

logging.exp('Using Site Configuration: %s' % config)
allowed_keys = list(itertools.chain.from_iterable(
    config['keys'].values()))
all_allowed_keys = allowed_keys + ['space']
left_allowed_keys = list(config['keys']['left'])
right_allowed_keys = list(config['keys']['right'])

runNumber = expInfo['runNumber']

if runNumber == 'practice':
    blocksList = 'conditions/practice1blocks.csv'
    runNumber = 0
    mode = 'practice'
else:  # session == '1'
    blocksList = 'conditions/run1blocks.csv'
    mode = 'scan'
    try:
        runNumber = int(runNumber)
    except ValueError:
        raise ValueError('runNumber must be either "practice" or a whole number, but it was "%s"\n' % runNumber)

titleLetterSize = config['style']['titleLetterSize']  # 3
subtitleLetterSize = config['style']['subtitleLetterSize']  # .7
textLetterSize = config['style']['textLetterSize']  # 1
fixLetterSize = config['style']['fixLetterSize']  # 2.5
wrapWidth = config['style']['wrapWidth']  # 30
subtitleLetterSize = config['style']['subtitleLetterSize']  # 1
frameRateRange = config['frameRateRange']  # [59, 121]

fixXOffset, fixYOffset = -.01, .1425

events = []  # run, condition, onset, duration, value

introText = visual.TextStim(win=win, ori=0, name='introText',
    text='CHECKER-\nBOARD\nGAME',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
outerFrame = visual.Rect(win=win, lineWidth=1, lineColor='white',
                         width=16.4, height=12.4, units='deg', name='outerFrame')

pointerXoffset = 5
pointerYoffset = -4.5
pointerLabelPrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='', pos=(-pointerXoffset, pointerYoffset), name='pointerLabelPrompt')
pointerFingerPrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='POINTER FINGER', pos=(-pointerXoffset, pointerYoffset-1), name='pointerFingerPrompt')
middleLabelPrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='', pos=(pointerXoffset, pointerYoffset), name='middleLabelPrompt')
middleFingerPrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='MIDDLE FINGER', pos=(pointerXoffset, pointerYoffset-1), name='middleFingerPrompt')

pointerLabelPrompt.text = 'Left'
middleLabelPrompt.text = 'Right'

buttonPrompts = [pointerLabelPrompt, pointerFingerPrompt, 
                 middleLabelPrompt, middleFingerPrompt]

# outerFrame.autoDraw = True
# Setup the RA Experimenter Window
raWin = visual.Window(size=[1280,800], fullscr=False, allowGUI=True, allowStencil=False,
    monitor=u'testMonitor', color=u'black', colorSpace='rgb',
    blendMode='avg', useFBO=True,
    units='deg',
    screen=0)

raText= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[0, -5], height=textLetterSize*1.5, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raVerbalText = visual.TextStim(win=raWin, ori=0, name='raVerbalText',
    text='', font='Arial',
    pos=[0,3], height=textLetterSize*1.5, wrapWidth=30,
    color='#3EB4F0', colorSpace='rgb', opacity=1,
    depth=0.0, italic=True)
def frameRateWarning():
    dlg = gui.Dlg(title='Frame Rate Warning')
    dlg.addText('WARNING: Frame rate seems too high or too low.')
    dlg.addText('Press "OK" to continue or "Cancel" to stop.')
    dlg.addFixedField('Measured Frame Rate', expInfo['frameRate'])
    dlg.addText('#'*40)
    dlg.addText('This may be caused by background processes or')
    dlg.addText('incorrect screen settings.')
    dlg.addText('')
    dlg.addText('If this is the first time seeing this message, you should:')
    dlg.addText('1. Cancel this run')
    dlg.addText('2. Close anything running in the background.')
    dlg.addText('3. Check that the screen refresh rate is correct.')
    dlg.addText('4. Re-run the script and try again.')
    dlg.addText('')
    return dlg


def checkRates(frameRateRange, statusTextStim):
    statusTextStim.text = 'Checking Screen Refresh - Please wait.'
    statusTextStim.draw()
    statusTextStim.win.flip()
    
    rates = [expInfo['frameRate']]
    for i in range(10):
        rates.append(win.getActualFrameRate())
    medianRate = np.median(rates)
    logging.exp('Measured Framerates: %s' % rates)
    if not (frameRateRange[0] < medianRate < frameRateRange[1]):
        logging.exp('Median Rate (%.03f) still out of range' % medianRate)
        dlg = frameRateWarning()
        dlg.show()
        if not dlg.OK:
            logging.exp('User Cancelled')
            core.quit()  # user pressed cancel
        else:
            logging.exp('User continued anyway; assuming 60Hz')
            # expInfo['frameRate'] = medianRate
            expInfo['frameRate'] = 60.0  # Assume a frame rate of 60
            expInfo['originalMedianFrameRate'] = medianRate
    else:
        logging.exp('Median Rate (%.03f) in acceptable range; continuing' % medianRate)
        expInfo['frameRate'] = medianRate

# Check for sane frame rate
if not (frameRateRange[0] < expInfo['frameRate'] < frameRateRange[1]):
    logging.exp('Unexpected frame rate was measured on first try: %.03f' %
                expInfo['frameRate'])
    checkRates(frameRateRange=frameRateRange, statusTextStim=raText)

frameDur = 1.0/round(expInfo['frameRate'])
# Record Git Status in Subject Logs
from subprocess import check_output, check_call, CalledProcessError
try:
    revision = check_output(['git','rev-parse', '--short', 'HEAD'])
    revision = revision.strip()
    gitInstalled = True
except:
    if os.path.exists('VERSION'):
        with open('VERSION', 'r') as f:
            revision = f.read().strip()
    else:
        revision = ''
    gitInstalled = False

if gitInstalled:
    try:
        check_call(['git', 'diff-files', '--quiet'])
    except CalledProcessError:
        revision = 'Task changes detected. Nearest head: %s' % revision
        status_msg = check_output(['git', 'status'])
        msg ="""%s

Warning: the experiment has unexpected changes.
""" % status_msg
        logging.critical(msg)
        # core.quit()

expInfo['git-revision'] = revision

import hcpcommon
movieClock = core.Clock()

record_example, cleanup = False, True

winRecorder = hcpcommon.WinRecorder(win=win, 
                                    clock=movieClock, 
                                    record=record_example,
                                    cleanup=cleanup)

raWinRecorder = hcpcommon.WinRecorder(win=raWin, 
                                      clock=movieClock, 
                                      record=record_example,
                                      cleanup=cleanup)

def save_movie(filename, recorder1, recorder2):
    win_fname = filename + '_win-movie.png'
    rawin_fname = filename + '_rawin-movie.png'

    # Save stills and convert to .mov
    recorder1.save(win_fname)
    recorder2.save(rawin_fname)
    movie_outfile = filename + '_EXAMPLE.mov'
    hcpcommon.WinRecorder.save_sidebyside([recorder1, recorder2], 
                                          movie_outfile)
    print('Saved %s' % movie_outfile)
    return movie_outfile


# Initialize components for Routine "directions1"
directions1Clock = core.Clock()
checkIma = visual.ImageStim(win=win, name='checkIma',
    image='images/checkerboard.png', mask=None,
    ori=0, pos=[0, 0], size=[12, 11.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=0.0)


# Initialize components for Routine "directions2"
directions2Clock = core.Clock()
checkIma2 = visual.ImageStim(win=win, name='checkIma2',
    image='images/checkerboard.png', mask=None,
    ori=0, pos=[0, 0], size=[12, 11.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=0.0)


# Initialize components for Routine "directions3"
directions3Clock = core.Clock()
directions3Text = visual.TextStim(win=win, ori=0, name='directions3Text',
    text='POINTER FINGER',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
directions3img = visual.ImageStim(win=win, name='directions3img',
    image='images/leftSquares.png', mask=None,
    ori=0, pos=[0,3], size=[6.5, 6.25],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
pointerFinger = visual.ImageStim(win=win, name='pointerFinger',
    image='images/pointerFingerButton.png', mask=None,
    ori=0, pos=[0, -2.5], size=[3.5, 3.75],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)

#imPracLeft = 'images/framesLeft.png'
#imPracRight = 'images/framesRight.png'


# Initialize components for Routine "directions3b"
directions3bClock = core.Clock()
directions3bText = visual.TextStim(win=win, ori=0, name='directions3bText',
    text='MIDDLE FINGER',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
directions3bimg = visual.ImageStim(win=win, name='directions3bimg',
    image='images/rightSquares.png', mask=None,
    ori=0, pos=[0,3], size=[6.5, 6.25],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
middleFinger = visual.ImageStim(win=win, name='middleFinger',
    image='images/middleFingerButton.png', mask=None,
    ori=0, pos=[0, -2.5], size=[3.5, 3.75],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)


# Initialize components for Routine "directions4"
directions4Clock = core.Clock()
directions4Text = visual.TextStim(win=win, ori=0, name='directions4Text',
    text=u'Be sure to make your choice and press the button as quickly as you can.\n\nLet\u2019s try an example now.',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)



# Initialize components for Routine "pracPretrialFix"
pracPretrialFixClock = core.Clock()
pracPretrialFixText = visual.TextStim(win=win, ori=0, name='pracPretrialFixText',
    text='+',    font='Arial',
    pos=[fixXOffset, fixYOffset], height=fixLetterSize, wrapWidth=None,
    color='green', colorSpace='rgb', opacity=1,
    depth=0.0)



# Initialize components for Routine "exampleTrial"
exampleTrialClock = core.Clock()

exCenterCircle = visual.Polygon(win=win, name='exCenterCircle',
    edges = 600, size=[2, 2],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor=[1,1,1], lineColorSpace='rgb',
    fillColor='black', fillColorSpace='rgb',
    opacity=1,depth=-2.0, 
interpolate=True)
exFixText = visual.TextStim(win=win, ori=0, name='exFixText',
    text='+',    font='Arial',
    pos=[fixXOffset, fixYOffset], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)



# Initialize components for Routine "feedback"
feedbackClock = core.Clock()
feedbackText = visual.TextStim(win=win, ori=0, name='feedbackText',
    text='default text',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "directions5"
directions5Clock = core.Clock()
directions5Text = visual.TextStim(win=win, ori=0, name='directions5Text',
    text='Do you have any questions?\n\nAre you ready to begin the practice? \n',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "directions2"
directions2Clock = core.Clock()
checkIma2 = visual.ImageStim(win=win, name='checkIma2',
    image='images/checkerboard.png', mask=None,
    ori=0, pos=[0, 0], size=[12, 11.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=0.0)


# Initialize components for Routine "directions3"
directions3Clock = core.Clock()
directions3Text = visual.TextStim(win=win, ori=0, name='directions3Text',
    text='POINTER FINGER',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
directions3img = visual.ImageStim(win=win, name='directions3img',
    image='images/leftSquares.png', mask=None,
    ori=0, pos=[0,3], size=[6.5, 6.25],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
pointerFinger = visual.ImageStim(win=win, name='pointerFinger',
    image='images/pointerFingerButton.png', mask=None,
    ori=0, pos=[0, -2.5], size=[3.5, 3.75],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)

#imPracLeft = 'images/framesLeft.png'
#imPracRight = 'images/framesRight.png'


# Initialize components for Routine "directions3b"
directions3bClock = core.Clock()
directions3bText = visual.TextStim(win=win, ori=0, name='directions3bText',
    text='MIDDLE FINGER',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
directions3bimg = visual.ImageStim(win=win, name='directions3bimg',
    image='images/rightSquares.png', mask=None,
    ori=0, pos=[0,3], size=[6.5, 6.25],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
middleFinger = visual.ImageStim(win=win, name='middleFinger',
    image='images/middleFingerButton.png', mask=None,
    ori=0, pos=[0, -2.5], size=[3.5, 3.75],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)


# Initialize components for Routine "runDirections3"
runDirections3Clock = core.Clock()
runDirections3Text = visual.TextStim(win=win, ori=0, name='runDirections3Text',
    text='Do you have any questions?',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "waitForScanner"
waitForScannerClock = core.Clock()
waitForScannerText = visual.TextStim(win=win, ori=0, name='waitForScannerText',
    text='Get ready!',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
import datetime

fmriClock = core.Clock()
trigger = 'usb'
#trigger = 'parallel'
if trigger == 'parallel':
    from psychopy.contrib import parallel as winioport
    #from psychopy import parallel
elif trigger == 'usb':
    from psychopy.hardware.emulator import launchScan
    #
    # settings for launchScan:
    MR_settings = { 
        'TR': 0, # duration (sec) per volume
        'volumes': 0, # number of whole-brain 3D volumes / frames
        'sync': config['trigger'], # character to use as the sync timing event; assumed to come at start of a volume
        'skip': 0, # number of volumes lacking a sync pulse at start of scan (for T1 stabilization)
        }




# Initialize components for Routine "countdown"
countdownClock = core.Clock()

countdownText = visual.TextStim(win=win, ori=0, name='countdownText',
    text='default text',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "fixRoutine"
fixRoutineClock = core.Clock()
practiceFixDur = 3
fixRoutineText = visual.TextStim(win=win, ori=0, name='fixRoutineText',
    text='+',    font='Arial',
    pos=[fixXOffset, fixYOffset], height=fixLetterSize, wrapWidth=None,
    color=1.0, colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "trial"
trialClock = core.Clock()
angularCycles = 20  # "Clean" cycles divisible by 360 include 20 and 30
radialCycles = 10
size = 10
baseOriOffset = 3  # To rotate the checkerboard so that the horizon is flat
angularRes = 1080
texRes = 64

angularCycles = 16  # "Clean" cycles divisible by 360 include 20 and 30
radialCycles = 6
baseOriOffset = 6
'''size = 20
 # To rotate the checkerboard so that the horizon is flat
angularRes = 32
texRes = 32
'''

flicker_frames = 0

numTargetSquares = 2
wedgeLength = 360 / (angularCycles * 2)
targetWedge = [0, wedgeLength * numTargetSquares]

def makeMask(targetCycle, totalCycles):
    a = np.zeros(totalCycles)
    a[targetCycle] = .8
    
    return a

def makeTarget(name, wedge):
    return visual.RadialStim(win, pos = [0, 0],tex='sqrXsqr',
        size=size, color='red', visibleWedge=wedge,
        ori=0, interpolate=False,
        radialCycles=0, angularCycles=0,
        autoLog=True, mask=None, angularRes=angularRes, texRes=texRes,
        contrast=-1, name=name)

checkerboard = visual.RadialStim(win, pos=[0, 0], tex='sqrXsqr',
    size=size, color=1, visibleWedge=[0, 360], 
    ori=baseOriOffset, interpolate=True,
    radialCycles=radialCycles, angularCycles=angularCycles, 
    autoLog=True, mask=None, angularRes=angularRes, texRes=texRes,
    name='checkerboard')

target1 = makeTarget('target1', wedge=targetWedge)
target2 = makeTarget('target2', wedge=targetWedge)

# # Framerate testing
# expInfo['frameRate'] = 85.012398432
# expInfo['frameRate'] = 15.0109384328
framesPerSecond = int(round(expInfo['frameRate']))
targetDur = .5  # Time (seconds) to display red target squares
targetFrames = int(framesPerSecond * targetDur)

flickerRate = 4  # Flash Rate in Hz. Rate is measured as a full
                 # cycle (black, white), so this will change 2x
                 # as fast.
flickerFramesPerHalfCycle = int(round(framesPerSecond / 
                                      (flickerRate * 2)))

logging.exp('Measured Frame Rate: %.05f' % expInfo['frameRate'])
logging.exp('Total Target Frames (500ms): %d' % targetFrames)
logging.exp('Checkerboard frames per half cycle (phase change): %d' % flickerFramesPerHalfCycle)

centerCircle = visual.Polygon(win=win, name='centerCircle',
    edges = 600, size=[2, 2],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor=[1,1,1], lineColorSpace='rgb',
    fillColor='black', fillColorSpace='rgb',
    opacity=1,depth=-4.0, 
interpolate=True)
fixText = visual.TextStim(win=win, ori=0, name='fixText',
    text='+',    font='Arial',
    pos=[fixXOffset, fixYOffset], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-5.0)


import pandas as pd
COLUMNS = [
    'trialNum', 
    'countdownStartTime', 'countdownLabel',
    'fixStartTime', 'fixEndTime', 'fixDur',
    'blockStartTime', 'blockEndTime',
    'targetStartTime', 'targetEndTime',
    'runEndTime',
    'side',
    'firstKey', 'firstRt', 'correct',
    'trialResp.keys', 'trialResp.rt',
    'earlyISI.keys', 'earlyISI.rt',
    'lateISI.keys', 'lateISI.rt',
    'feedback', 'resp', 'respLast', 'noResp',
    'correctResponse', 'leftCount', 'rightCount', 
    'leftCorrCount', 'rightCorrCount', 
    'leftAcc', 'rightAcc', 'totalAcc', 
    'totalFiveBackAvg', 'countConsecutive',
    'trialsList',
] + expInfo.keys()

def updateRunLog(entries):
    df = pd.DataFrame(entries)

    # Filter rows for this run and selected columns
    run_df = df.loc[df['runs.thisN'] == runs.thisN].filter(COLUMNS)
    run_df['run'] = runNum  # 1-based index for "run"

    run_df.dropna(axis='index', subset=['trialNum'], inplace=True)
    run_df['trialNum'] = run_df['trialNum'].astype(int)

    runOutfile = '%s_wide.csv' % filename
    run_df.to_csv(runOutfile, encoding='utf-8',
                  header=True, index=False, )


# Initialize components for Routine "endFixation"
endFixationClock = core.Clock()
endFixationText = visual.TextStim(win=win, ori=0, name='endFixationText',
    text='+',    font='Arial',
    pos=[fixXOffset, fixYOffset], height=fixLetterSize, wrapWidth=None,
    color='green', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "endRun"
endRunClock = core.Clock()
if mode == 'practice':
    endRunMsg = 'You finished the practice!'
else:
    endRunMsg = 'You finished the session!'

endRunText = visual.TextStim(win=win, ori=0, name='endRunText',
    text=endRunMsg,    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Create some handy timers
globalClock = core.Clock()  # to track the time since experiment started
routineTimer = core.CountdownTimer()  # to track time remaining of each (non-slip) routine 

#------Prepare to start Routine "intro"-------
t = 0
introClock.reset()  # clock 
frameN = -1
# update component parameters for each repeat

introResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
introResp.status = NOT_STARTED

raVerbalText.text='"Welcome to the Checkerboard Game!"'
raText.text='Press <space> to continue.'

raVerbalText.draw()
raText.draw()
raWinRecorder.keyframe()
raWin.flip()


movieClock.reset()
# keep track of which components have finished
introComponents = []
introComponents.append(introText)
introComponents.append(introResp)
for thisComponent in introComponents:
    if hasattr(thisComponent, 'status'):
        thisComponent.status = NOT_STARTED

#-------Start Routine "intro"-------
continueRoutine = True
while continueRoutine:
    # get current time
    t = introClock.getTime()
    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
    # update/draw components on each frame
    
    
    # *introText* updates
    if t >= 0.0 and introText.status == NOT_STARTED:
        # keep track of start time/frame for later
        introText.tStart = t  # underestimates by a little under one frame
        introText.frameNStart = frameN  # exact frame index
        introText.setAutoDraw(True)
    
    # *introResp* updates
    if t >= 0.0 and introResp.status == NOT_STARTED:
        # keep track of start time/frame for later
        introResp.tStart = t  # underestimates by a little under one frame
        introResp.frameNStart = frameN  # exact frame index
        introResp.status = STARTED
        # keyboard checking is just starting
        event.clearEvents(eventType='keyboard')
    if introResp.status == STARTED:
        theseKeys = event.getKeys(keyList=['space'])
        
        # check for quit:
        if "escape" in theseKeys:
            endExpNow = True
        if len(theseKeys) > 0:  # at least one key was pressed
            # a response ends the routine
            continueRoutine = False
    
    
    
    
    if frameN == 0:
        winRecorder.keyframe()
    
    # check if all components have finished
    if not continueRoutine:  # a component has requested a forced-end of Routine
        break
    continueRoutine = False  # will revert to True if at least one component still running
    for thisComponent in introComponents:
        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
            continueRoutine = True
            break  # at least one component has not yet finished
    
    # check for quit (the Esc key)
    if endExpNow or event.getKeys(keyList=["escape"]):
        core.quit()
    
    # refresh the screen
    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
        win.flip()

#-------Ending Routine "intro"-------
for thisComponent in introComponents:
    if hasattr(thisComponent, "setAutoDraw"):
        thisComponent.setAutoDraw(False)






# the Routine "intro" was not non-slip safe, so reset the non-slip timer
routineTimer.reset()

# set up handler to look after randomisation of conditions etc
practiceInstructions = data.TrialHandler(nReps=mode == 'practice', method='random', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='practiceInstructions')
thisExp.addLoop(practiceInstructions)  # add the loop to the experiment
thisPracticeInstruction = practiceInstructions.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisPracticeInstruction.rgb)
if thisPracticeInstruction != None:
    for paramName in thisPracticeInstruction.keys():
        exec(paramName + '= thisPracticeInstruction.' + paramName)

for thisPracticeInstruction in practiceInstructions:
    currentLoop = practiceInstructions
    # abbreviate parameter names if possible (e.g. rgb = thisPracticeInstruction.rgb)
    if thisPracticeInstruction != None:
        for paramName in thisPracticeInstruction.keys():
            exec(paramName + '= thisPracticeInstruction.' + paramName)
    
    #------Prepare to start Routine "directions1"-------
    t = 0
    directions1Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    directions1Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    directions1Resp.status = NOT_STARTED
    raVerbalText.text='"In this task, you will see a circular checkerboard, like this one. \n\nAs you can see here, there will be a plus sign in the center of the screen. Please look at the plus sign during the whole task, keeping your eyes relaxed and blinking normally."'
    raText.text='Press <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    directions1Components = []
    directions1Components.append(checkIma)
    directions1Components.append(directions1Resp)
    for thisComponent in directions1Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions1"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions1Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *checkIma* updates
        if t >= 0.0 and checkIma.status == NOT_STARTED:
            # keep track of start time/frame for later
            checkIma.tStart = t  # underestimates by a little under one frame
            checkIma.frameNStart = frameN  # exact frame index
            checkIma.setAutoDraw(True)
        
        # *directions1Resp* updates
        if t >= 0.0 and directions1Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions1Resp.tStart = t  # underestimates by a little under one frame
            directions1Resp.frameNStart = frameN  # exact frame index
            directions1Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(directions1Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if directions1Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                directions1Resp.keys = theseKeys[-1]  # just the last key pressed
                directions1Resp.rt = directions1Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions1Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions1"-------
    for thisComponent in directions1Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if directions1Resp.keys in ['', [], None]:  # No response was made
       directions1Resp.keys=None
    # store data for practiceInstructions (TrialHandler)
    practiceInstructions.addData('directions1Resp.keys',directions1Resp.keys)
    if directions1Resp.keys != None:  # we had a response
        practiceInstructions.addData('directions1Resp.rt', directions1Resp.rt)
    
    # the Routine "directions1" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directions2"-------
    t = 0
    directions2Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    directions2Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    directions2Resp.status = NOT_STARTED
    if mode =='practice':
        raVerbalText.text='"The squares will flicker between black and white. Occasionally, some squares will turn red very briefly.\n\nYour job is to respond quickly whether the RED squares are on the LEFT or RIGHT side of the checkerboard."'
    else:
        raVerbalText.text='"As a reminder, the squares in the checkerboard will flicker between black and white. Occasionally, some squares will turn red very briefly.\n\nYour job is to respond quickly whether the RED squares are on the LEFT or RIGHT side of the checkerboard."'
    
    raText.text='Press <space> to continue.'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    directions2Components = []
    directions2Components.append(checkIma2)
    directions2Components.append(directions2Resp)
    for thisComponent in directions2Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions2"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions2Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *checkIma2* updates
        if t >= 0.0 and checkIma2.status == NOT_STARTED:
            # keep track of start time/frame for later
            checkIma2.tStart = t  # underestimates by a little under one frame
            checkIma2.frameNStart = frameN  # exact frame index
            checkIma2.setAutoDraw(True)
        
        # *directions2Resp* updates
        if t >= 0.0 and directions2Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions2Resp.tStart = t  # underestimates by a little under one frame
            directions2Resp.frameNStart = frameN  # exact frame index
            directions2Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(directions2Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if directions2Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                directions2Resp.keys = theseKeys[-1]  # just the last key pressed
                directions2Resp.rt = directions2Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions2Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions2"-------
    for thisComponent in directions2Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if directions2Resp.keys in ['', [], None]:  # No response was made
       directions2Resp.keys=None
    # store data for practiceInstructions (TrialHandler)
    practiceInstructions.addData('directions2Resp.keys',directions2Resp.keys)
    if directions2Resp.keys != None:  # we had a response
        practiceInstructions.addData('directions2Resp.rt', directions2Resp.rt)
    
    # the Routine "directions2" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directions3"-------
    t = 0
    directions3Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    dir3Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    dir3Resp.status = NOT_STARTED
    raVerbalText.text='"When the RED squares are on the LEFT, press quickly with your POINTER finger. Try pressing with your POINTER finger now."'
    raText.text='\n\nWait for participant to respond to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    #npresses = 0
    #maxPresses = 2
    
    
    # keep track of which components have finished
    directions3Components = []
    directions3Components.append(directions3Text)
    directions3Components.append(directions3img)
    directions3Components.append(pointerFinger)
    directions3Components.append(dir3Resp)
    for thisComponent in directions3Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions3"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions3Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *directions3Text* updates
        if t >= 0.0 and directions3Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3Text.tStart = t  # underestimates by a little under one frame
            directions3Text.frameNStart = frameN  # exact frame index
            directions3Text.setAutoDraw(True)
        
        # *directions3img* updates
        if t >= 0.0 and directions3img.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3img.tStart = t  # underestimates by a little under one frame
            directions3img.frameNStart = frameN  # exact frame index
            directions3img.setAutoDraw(True)
        
        # *pointerFinger* updates
        if t >= 0.0 and pointerFinger.status == NOT_STARTED:
            # keep track of start time/frame for later
            pointerFinger.tStart = t  # underestimates by a little under one frame
            pointerFinger.frameNStart = frameN  # exact frame index
            pointerFinger.setAutoDraw(True)
        
        # *dir3Resp* updates
        if t >= 0.0 and dir3Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            dir3Resp.tStart = t  # underestimates by a little under one frame
            dir3Resp.frameNStart = frameN  # exact frame index
            dir3Resp.status = STARTED
            # AllowedKeys looks like a variable named `left_allowed_keys`
            if not 'left_allowed_keys' in locals():
                logging.error('AllowedKeys variable `left_allowed_keys` is not defined.')
                core.quit()
            if not type(left_allowed_keys) in [list, tuple, np.ndarray]:
                if not isinstance(left_allowed_keys, basestring):
                    logging.error('AllowedKeys variable `left_allowed_keys` is not string- or list-like.')
                    core.quit()
                elif not ',' in left_allowed_keys: left_allowed_keys = (left_allowed_keys,)
                else:  left_allowed_keys = eval(left_allowed_keys)
            # keyboard checking is just starting
            win.callOnFlip(dir3Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if dir3Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=list(left_allowed_keys))
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                dir3Resp.keys = theseKeys[-1]  # just the last key pressed
                dir3Resp.rt = dir3Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        '''if event.getKeys(keyList = all_allowed_keys):
            npresses += 1
            if npresses == maxPresses:
                continueRoutine = False
            if npresses == 1:
                directions3img.image = imPracRight
                directions3Text.text = 'MIDDLE FINGER'
        '''
        '''
        if npresses==0 and event.getKeys(keyList = left_allowed_keys):
            npresses+=1
            directions3img.image = imPracRight
            directions3Text.text = 'MIDDLE FINGER'
        if npresses==1 and event.getKeys(keyList = right_allowed_keys):
            continueRoutine = False
        '''
        #    if npresses == 3:
        #        directions4img.image = imPrac4a
        #        directions4Text.text = 'Press the button under your index finger to indicate that this picture is "NEW".'
        #    if npresses == 4:
        #        directions4img.image = imPrac5a
        #        directions4Text.text = 'Press the button under your index finger to indicate that this picture is "NEW".'
        #    if npresses == 5:
        #        directions4img.image = imPrac2a
        #        directions4Text.text = 'This picture is exactly the same as one that you saw earlier. \n\nSo, for this picture, press the button under your middle finger to indicate that it is "OLD".'
        #    if npresses == 6:
        #        directions4img.image = imPrac1b
        #        directions4Text.text = 'This picture looks similar to one you saw earlier.'
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions3Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions3"-------
    for thisComponent in directions3Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if dir3Resp.keys in ['', [], None]:  # No response was made
       dir3Resp.keys=None
    # store data for practiceInstructions (TrialHandler)
    practiceInstructions.addData('dir3Resp.keys',dir3Resp.keys)
    if dir3Resp.keys != None:  # we had a response
        practiceInstructions.addData('dir3Resp.rt', dir3Resp.rt)
    
    
    # the Routine "directions3" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directions3b"-------
    t = 0
    directions3bClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    dir3bResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    dir3bResp.status = NOT_STARTED
    raVerbalText.text='"When the RED squares are on the RIGHT, press quickly with your MIDDLE finger. Try pressing with your MIDDLE finger now."'
    raText.text='\n\nWait for participant to respond to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    
    # keep track of which components have finished
    directions3bComponents = []
    directions3bComponents.append(directions3bText)
    directions3bComponents.append(directions3bimg)
    directions3bComponents.append(middleFinger)
    directions3bComponents.append(dir3bResp)
    for thisComponent in directions3bComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions3b"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions3bClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *directions3bText* updates
        if t >= 0.0 and directions3bText.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3bText.tStart = t  # underestimates by a little under one frame
            directions3bText.frameNStart = frameN  # exact frame index
            directions3bText.setAutoDraw(True)
        
        # *directions3bimg* updates
        if t >= 0.0 and directions3bimg.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3bimg.tStart = t  # underestimates by a little under one frame
            directions3bimg.frameNStart = frameN  # exact frame index
            directions3bimg.setAutoDraw(True)
        
        # *middleFinger* updates
        if t >= 0.0 and middleFinger.status == NOT_STARTED:
            # keep track of start time/frame for later
            middleFinger.tStart = t  # underestimates by a little under one frame
            middleFinger.frameNStart = frameN  # exact frame index
            middleFinger.setAutoDraw(True)
        
        # *dir3bResp* updates
        if t >= 0.0 and dir3bResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            dir3bResp.tStart = t  # underestimates by a little under one frame
            dir3bResp.frameNStart = frameN  # exact frame index
            dir3bResp.status = STARTED
            # AllowedKeys looks like a variable named `right_allowed_keys`
            if not 'right_allowed_keys' in locals():
                logging.error('AllowedKeys variable `right_allowed_keys` is not defined.')
                core.quit()
            if not type(right_allowed_keys) in [list, tuple, np.ndarray]:
                if not isinstance(right_allowed_keys, basestring):
                    logging.error('AllowedKeys variable `right_allowed_keys` is not string- or list-like.')
                    core.quit()
                elif not ',' in right_allowed_keys: right_allowed_keys = (right_allowed_keys,)
                else:  right_allowed_keys = eval(right_allowed_keys)
            # keyboard checking is just starting
            win.callOnFlip(dir3bResp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if dir3bResp.status == STARTED:
            theseKeys = event.getKeys(keyList=list(right_allowed_keys))
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                dir3bResp.keys = theseKeys[-1]  # just the last key pressed
                dir3bResp.rt = dir3bResp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions3bComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions3b"-------
    for thisComponent in directions3bComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if dir3bResp.keys in ['', [], None]:  # No response was made
       dir3bResp.keys=None
    # store data for practiceInstructions (TrialHandler)
    practiceInstructions.addData('dir3bResp.keys',dir3bResp.keys)
    if dir3bResp.keys != None:  # we had a response
        practiceInstructions.addData('dir3bResp.rt', dir3bResp.rt)
    
    # the Routine "directions3b" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directions4"-------
    t = 0
    directions4Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    directionsResp4 = event.BuilderKeyResponse()  # create an object of type KeyResponse
    directionsResp4.status = NOT_STARTED
    side = 'left'  # Practice Trial
    raVerbalText.text='"Be sure to make your choice and press the button as quickly as you can. \n\nLet\'s try an example now."'
    raText.text='Press <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    directions4Components = []
    directions4Components.append(directions4Text)
    directions4Components.append(directionsResp4)
    for thisComponent in directions4Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions4"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions4Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *directions4Text* updates
        if t >= 0.0 and directions4Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions4Text.tStart = t  # underestimates by a little under one frame
            directions4Text.frameNStart = frameN  # exact frame index
            directions4Text.setAutoDraw(True)
        
        # *directionsResp4* updates
        if t >= 0.0 and directionsResp4.status == NOT_STARTED:
            # keep track of start time/frame for later
            directionsResp4.tStart = t  # underestimates by a little under one frame
            directionsResp4.frameNStart = frameN  # exact frame index
            directionsResp4.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(directionsResp4.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if directionsResp4.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                directionsResp4.keys = theseKeys[-1]  # just the last key pressed
                directionsResp4.rt = directionsResp4.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions4Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions4"-------
    for thisComponent in directions4Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if directionsResp4.keys in ['', [], None]:  # No response was made
       directionsResp4.keys=None
    # store data for practiceInstructions (TrialHandler)
    practiceInstructions.addData('directionsResp4.keys',directionsResp4.keys)
    if directionsResp4.keys != None:  # we had a response
        practiceInstructions.addData('directionsResp4.rt', directionsResp4.rt)
    
    
    # the Routine "directions4" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "pracPretrialFix"-------
    t = 0
    pracPretrialFixClock.reset()  # clock 
    frameN = -1
    routineTimer.add(1.500000)
    # update component parameters for each repeat
    fixDur = 2.25  # Fixation for first trials
    fixFrames = int(framesPerSecond * fixDur)
    
    noResp=0
    
    leftCount = 0
    rightCount=0
    
    leftCorrCount=0
    rightCorrCount=0
    
    totalCorrCount=0
    
    leftAcc=0
    rightAcc=0
    totalAcc=0
    
    totalAccList=[]
    totalFiveBackAvg = 0
    raText.pos=[0,0]
    raText.text = 'Watch the participant\'s performance and provide feedback as appropriate.'
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    pracPretrialFixComponents = []
    pracPretrialFixComponents.append(pracPretrialFixText)
    for thisComponent in pracPretrialFixComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "pracPretrialFix"-------
    continueRoutine = True
    while continueRoutine and routineTimer.getTime() > 0:
        # get current time
        t = pracPretrialFixClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *pracPretrialFixText* updates
        if t >= 0.0 and pracPretrialFixText.status == NOT_STARTED:
            # keep track of start time/frame for later
            pracPretrialFixText.tStart = t  # underestimates by a little under one frame
            pracPretrialFixText.frameNStart = frameN  # exact frame index
            pracPretrialFixText.setAutoDraw(True)
        if pracPretrialFixText.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
            pracPretrialFixText.setAutoDraw(False)
        
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in pracPretrialFixComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "pracPretrialFix"-------
    for thisComponent in pracPretrialFixComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    
    # set up handler to look after randomisation of conditions etc
    exampleTrials = data.TrialHandler(nReps=2, method='random', 
        extraInfo=expInfo, originPath=-1,
        trialList=[None],
        seed=None, name='exampleTrials')
    thisExp.addLoop(exampleTrials)  # add the loop to the experiment
    thisExampleTrial = exampleTrials.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisExampleTrial.rgb)
    if thisExampleTrial != None:
        for paramName in thisExampleTrial.keys():
            exec(paramName + '= thisExampleTrial.' + paramName)
    
    for thisExampleTrial in exampleTrials:
        currentLoop = exampleTrials
        # abbreviate parameter names if possible (e.g. rgb = thisExampleTrial.rgb)
        if thisExampleTrial != None:
            for paramName in thisExampleTrial.keys():
                exec(paramName + '= thisExampleTrial.' + paramName)
        
        #------Prepare to start Routine "exampleTrial"-------
        t = 0
        exampleTrialClock.reset()  # clock 
        frameN = -1
        routineTimer.add(3.000000)
        # update component parameters for each repeat
        if side == 'right':
            angularOffset = 5  # 7
        else:  # side == 'left'
            angularOffset = 21  # 27
        
        radialOut = 6
        
        targetOri = baseOriOffset + ((wedgeLength / 2) + wedgeLength * angularOffset)
        radialMask = makeMask([radialOut, radialOut+1], radialCycles * 2)
        target1.mask = radialMask
        target1.ori = targetOri
        
        ##angularOffset = 18
        targetOri = baseOriOffset + ((wedgeLength / 2) + wedgeLength * (angularOffset + 2))
        radialMask = makeMask([radialOut+2, radialOut+3], radialCycles * 2)
        target2.mask = radialMask
        target2.ori = targetOri
        
        #win.recordFrameIntervals = True
        
        # Number of frames to wait before starting target (ISI)
        # minus 3 frames worth of slop (current time to write checkerboard and start trial)
        fixFrames = int(framesPerSecond * fixDur) - 4
        
        #trialCountdown.add(fixDur + targetDur)  # 3 s duration
        exampleTrialResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
        exampleTrialResp.status = NOT_STARTED
        number_trials = exampleTrials.thisN + 1
        raText.text = 'Watch the participant\'s performance and provide feedback as appropriate.'
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
        
        record_expt_win = True
        # keep track of which components have finished
        exampleTrialComponents = []
        exampleTrialComponents.append(exampleTrialResp)
        exampleTrialComponents.append(exCenterCircle)
        exampleTrialComponents.append(exFixText)
        for thisComponent in exampleTrialComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "exampleTrial"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = exampleTrialClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            for prompt in buttonPrompts:
                prompt.draw()
            
            #logging.info('t: %02.2f' % t)
            
            #if frameN >= (fixFrames + targetFrames):
            #    continueRoutine = False
            
            flicker_frames += 1
            if flicker_frames >= 7:
                checkerboard.contrast = checkerboard.contrast * -1
                flicker_frames = 0
                record_expt_win = True
            
            # Draw the background first
            checkerboard.draw()
            
            if (frameN > fixFrames) and (frameN < (fixFrames + targetFrames)) and continueRoutine:
                target1.draw()
                target2.draw()
                if target1.status != STARTED and currentLoop.name == 'exampleTrials':
                        exampleTrials.addData('targetStart', fmriClock.getTime())
                target1.status = STARTED
                target2.status = STARTED
            
            #  '%02.2f, %d' % (t, frameN)
            if frameN == 0 and currentLoop.name == 'exampleTrials':
                exampleTrials.addData('checkerboardStart', fmriClock.getTime())
                exampleTrialComponents.extend([target1, target2])
            
            # *exampleTrialResp* updates
            if t >= 2.5 and exampleTrialResp.status == NOT_STARTED:
                # keep track of start time/frame for later
                exampleTrialResp.tStart = t  # underestimates by a little under one frame
                exampleTrialResp.frameNStart = frameN  # exact frame index
                exampleTrialResp.status = STARTED
                # AllowedKeys looks like a variable named `allowed_keys`
                if not 'allowed_keys' in locals():
                    logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                    core.quit()
                if not type(allowed_keys) in [list, tuple, np.ndarray]:
                    if not isinstance(allowed_keys, basestring):
                        logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                        core.quit()
                    elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                    else:  allowed_keys = eval(allowed_keys)
                # keyboard checking is just starting
                win.callOnFlip(exampleTrialResp.clock.reset)  # t=0 on next screen flip
                event.clearEvents(eventType='keyboard')
            if exampleTrialResp.status == STARTED and t >= (2.5 + (.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                exampleTrialResp.status = STOPPED
            if exampleTrialResp.status == STARTED:
                theseKeys = event.getKeys(keyList=list(allowed_keys))
                
                # check for quit:
                if "escape" in theseKeys:
                    endExpNow = True
                if len(theseKeys) > 0:  # at least one key was pressed
                    exampleTrialResp.keys.extend(theseKeys)  # storing all keys
                    exampleTrialResp.rt.append(exampleTrialResp.clock.getTime())
            
            # *exCenterCircle* updates
            if t >= 0 and exCenterCircle.status == NOT_STARTED:
                # keep track of start time/frame for later
                exCenterCircle.tStart = t  # underestimates by a little under one frame
                exCenterCircle.frameNStart = frameN  # exact frame index
                exCenterCircle.setAutoDraw(True)
            if exCenterCircle.status == STARTED and t >= (0 + (3-win.monitorFramePeriod*0.75)): #most of one frame period left
                exCenterCircle.setAutoDraw(False)
            
            # *exFixText* updates
            if t >= 0.0 and exFixText.status == NOT_STARTED:
                # keep track of start time/frame for later
                exFixText.tStart = t  # underestimates by a little under one frame
                exFixText.frameNStart = frameN  # exact frame index
                exFixText.setAutoDraw(True)
            if exFixText.status == STARTED and t >= (0.0 + (3-win.monitorFramePeriod*0.75)): #most of one frame period left
                exFixText.setAutoDraw(False)
            
            if record_expt_win:
                winRecorder.keyframe()
                record_expt_win = False
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in exampleTrialComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "exampleTrial"-------
        for thisComponent in exampleTrialComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        #win.saveFrameIntervals('/Users/kastman/Desktop/Intervals.log', clear=False)
        currentLoop.addData('endTrial', fmriClock.getTime())
        currentLoop.addData('correct', exampleTrialResp.keys in config['keys'][side])
        target1.status = NOT_STARTED
        target2.status = NOT_STARTED
        # check responses
        if exampleTrialResp.keys in ['', [], None]:  # No response was made
           exampleTrialResp.keys=None
        # store data for exampleTrials (TrialHandler)
        exampleTrials.addData('exampleTrialResp.keys',exampleTrialResp.keys)
        if exampleTrialResp.keys != None:  # we had a response
            exampleTrials.addData('exampleTrialResp.rt', exampleTrialResp.rt)
        resp=None
        respLast=None
        
        if exampleTrialResp.keys:
            if (exampleTrialResp.keys[0] in config['keys']['left'] and side == 'left'): #correct left
                resp='left'
                feedbackMsg = 'CORRECT'
                correct_response = 1
                leftCount+=1
                leftCorrCount += 1
                totalCorrCount += 1
                leftAcc= (leftCorrCount)/(leftCount)
            elif (exampleTrialResp.keys[0] in config['keys']['right'] and side == 'right'): #correct right
                resp='right'
                feedbackMsg = 'CORRECT'
                correct_response = 1
                rightCount+=1
                rightCorrCount += 1
                totalCorrCount += 1
                rightAcc= (rightCorrCount)/(rightCount)
            elif (exampleTrialResp.keys[0] in config['keys']['right'] and side == 'left'): #pressed right to left
                resp='right'
                feedbackMsg = 'INCORRECT'
                correct_response = 0
                leftCount+=1
                leftAcc= (leftCorrCount)/(leftCount)
            elif (exampleTrialResp.keys[0] in config['keys']['left'] and side == 'right'): #pressed left to right
                resp='left'
                feedbackMsg = 'INCORRECT'
                correct_response = 0
                rightCount+=1
                rightAcc= (rightCorrCount)/(rightCount)
        elif (not exampleTrialResp.keys and side == 'left'): #no response
                feedbackMsg = 'TOO SLOW'
                correct_response = 0
                noResp+=1
                leftCount+=1
                leftAcc= (leftCorrCount)/(leftCount)
        elif (not exampleTrialResp.keys and side == 'right'): #no response
                feedbackMsg = 'TOO SLOW'
                correct_response = 0
                noResp+=1
                rightCount+=1
                rightAcc= (rightCorrCount)/(rightCount)
        
        totalAccList.append(correct_response)
        totalAcc=(totalCorrCount)/(number_trials)
        
        #coding last button press
        if exampleTrialResp.keys:
            if exampleTrialResp.keys[-1] in config['keys']['left']:
                respLast='left'
            elif exampleTrialResp.keys[-1] in config['keys']['right']:
                respLast='right'
        
        
        
        #TotalAcc5back
        if number_trials >= 5:
            totalFiveBackAvg = 0
            for i in range(-1, -6,-1):
                totalFiveBackAvg+=totalAccList[i]
            totalFiveBackAvg=totalFiveBackAvg/5.0
        
        
        currentLoop.addData('feedback', feedbackMsg)
        
        currentLoop.addData('resp', resp)
        currentLoop.addData('respLast', respLast)
        currentLoop.addData('noResp', noResp)
        
        currentLoop.addData('correctResponse', correct_response)
        
        currentLoop.addData('leftCount', leftCount)
        currentLoop.addData('rightCount', rightCount)
        
        currentLoop.addData('leftCorrCount', leftCorrCount)
        currentLoop.addData('rightCorrCount', rightCorrCount)
        
        currentLoop.addData('leftAcc', leftAcc)
        currentLoop.addData('rightAcc', rightAcc)
        currentLoop.addData('totalAcc', totalAcc)
        
        currentLoop.addData('totalAccList', totalAccList)
        currentLoop.addData('totalFiveBackAvg', totalFiveBackAvg)
        
        
        
        #------Prepare to start Routine "feedback"-------
        t = 0
        feedbackClock.reset()  # clock 
        frameN = -1
        routineTimer.add(1.000000)
        # update component parameters for each repeat
        feedbackText.setText(feedbackMsg)
        record_expt_win = True
        # keep track of which components have finished
        feedbackComponents = []
        feedbackComponents.append(feedbackText)
        for thisComponent in feedbackComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "feedback"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = feedbackClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            # *feedbackText* updates
            if t >= 0.0 and feedbackText.status == NOT_STARTED:
                # keep track of start time/frame for later
                feedbackText.tStart = t  # underestimates by a little under one frame
                feedbackText.frameNStart = frameN  # exact frame index
                feedbackText.setAutoDraw(True)
            if feedbackText.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                feedbackText.setAutoDraw(False)
            for prompt in buttonPrompts:
                prompt.draw()
            
            if record_expt_win:
                winRecorder.keyframe()
                record_expt_win = False
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "feedback"-------
        for thisComponent in feedbackComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        
        thisExp.nextEntry()
        
    # completed 2 repeats of 'exampleTrials'
    
    
    #------Prepare to start Routine "directions5"-------
    t = 0
    directions5Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    raText.pos=[0,-4]
    raVerbalText.text = '"Now you will complete a short practice that will look exactly like the game you will play in the scanner.\n\nIn this practice, we will not tell you if you are right or wrong. Just try your best to follow the instructions and play the game.\n\nAre you ready to begin the practice?"'
    raText.text= '\n\nPress <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    dir5resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    dir5resp.status = NOT_STARTED
    # keep track of which components have finished
    directions5Components = []
    directions5Components.append(directions5Text)
    directions5Components.append(dir5resp)
    for thisComponent in directions5Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions5"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions5Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *directions5Text* updates
        if t >= 0 and directions5Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions5Text.tStart = t  # underestimates by a little under one frame
            directions5Text.frameNStart = frameN  # exact frame index
            directions5Text.setAutoDraw(True)
        if frameN == 0:
            winRecorder.keyframe()
        
        # *dir5resp* updates
        if t >= 0.0 and dir5resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            dir5resp.tStart = t  # underestimates by a little under one frame
            dir5resp.frameNStart = frameN  # exact frame index
            dir5resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(dir5resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if dir5resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                dir5resp.keys = theseKeys[-1]  # just the last key pressed
                dir5resp.rt = dir5resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions5Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions5"-------
    for thisComponent in directions5Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    raText.text=''
    raText.pos = [0, 0]
    raText.draw()
    raWin.flip()
    # check responses
    if dir5resp.keys in ['', [], None]:  # No response was made
       dir5resp.keys=None
    # store data for practiceInstructions (TrialHandler)
    practiceInstructions.addData('dir5resp.keys',dir5resp.keys)
    if dir5resp.keys != None:  # we had a response
        practiceInstructions.addData('dir5resp.rt', dir5resp.rt)
    # the Routine "directions5" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed mode == 'practice' repeats of 'practiceInstructions'


# set up handler to look after randomisation of conditions etc
runInstructions = data.TrialHandler(nReps=mode != 'practice', method='random', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='runInstructions')
thisExp.addLoop(runInstructions)  # add the loop to the experiment
thisRunInstruction = runInstructions.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisRunInstruction.rgb)
if thisRunInstruction != None:
    for paramName in thisRunInstruction.keys():
        exec(paramName + '= thisRunInstruction.' + paramName)

for thisRunInstruction in runInstructions:
    currentLoop = runInstructions
    # abbreviate parameter names if possible (e.g. rgb = thisRunInstruction.rgb)
    if thisRunInstruction != None:
        for paramName in thisRunInstruction.keys():
            exec(paramName + '= thisRunInstruction.' + paramName)
    
    #------Prepare to start Routine "directions2"-------
    t = 0
    directions2Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    directions2Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    directions2Resp.status = NOT_STARTED
    if mode =='practice':
        raVerbalText.text='"The squares will flicker between black and white. Occasionally, some squares will turn red very briefly.\n\nYour job is to respond quickly whether the RED squares are on the LEFT or RIGHT side of the checkerboard."'
    else:
        raVerbalText.text='"As a reminder, the squares in the checkerboard will flicker between black and white. Occasionally, some squares will turn red very briefly.\n\nYour job is to respond quickly whether the RED squares are on the LEFT or RIGHT side of the checkerboard."'
    
    raText.text='Press <space> to continue.'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    directions2Components = []
    directions2Components.append(checkIma2)
    directions2Components.append(directions2Resp)
    for thisComponent in directions2Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions2"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions2Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *checkIma2* updates
        if t >= 0.0 and checkIma2.status == NOT_STARTED:
            # keep track of start time/frame for later
            checkIma2.tStart = t  # underestimates by a little under one frame
            checkIma2.frameNStart = frameN  # exact frame index
            checkIma2.setAutoDraw(True)
        
        # *directions2Resp* updates
        if t >= 0.0 and directions2Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions2Resp.tStart = t  # underestimates by a little under one frame
            directions2Resp.frameNStart = frameN  # exact frame index
            directions2Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(directions2Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if directions2Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                directions2Resp.keys = theseKeys[-1]  # just the last key pressed
                directions2Resp.rt = directions2Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions2Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions2"-------
    for thisComponent in directions2Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if directions2Resp.keys in ['', [], None]:  # No response was made
       directions2Resp.keys=None
    # store data for runInstructions (TrialHandler)
    runInstructions.addData('directions2Resp.keys',directions2Resp.keys)
    if directions2Resp.keys != None:  # we had a response
        runInstructions.addData('directions2Resp.rt', directions2Resp.rt)
    
    # the Routine "directions2" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directions3"-------
    t = 0
    directions3Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    dir3Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    dir3Resp.status = NOT_STARTED
    raVerbalText.text='"When the RED squares are on the LEFT, press quickly with your POINTER finger. Try pressing with your POINTER finger now."'
    raText.text='\n\nWait for participant to respond to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    #npresses = 0
    #maxPresses = 2
    
    
    # keep track of which components have finished
    directions3Components = []
    directions3Components.append(directions3Text)
    directions3Components.append(directions3img)
    directions3Components.append(pointerFinger)
    directions3Components.append(dir3Resp)
    for thisComponent in directions3Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions3"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions3Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *directions3Text* updates
        if t >= 0.0 and directions3Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3Text.tStart = t  # underestimates by a little under one frame
            directions3Text.frameNStart = frameN  # exact frame index
            directions3Text.setAutoDraw(True)
        
        # *directions3img* updates
        if t >= 0.0 and directions3img.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3img.tStart = t  # underestimates by a little under one frame
            directions3img.frameNStart = frameN  # exact frame index
            directions3img.setAutoDraw(True)
        
        # *pointerFinger* updates
        if t >= 0.0 and pointerFinger.status == NOT_STARTED:
            # keep track of start time/frame for later
            pointerFinger.tStart = t  # underestimates by a little under one frame
            pointerFinger.frameNStart = frameN  # exact frame index
            pointerFinger.setAutoDraw(True)
        
        # *dir3Resp* updates
        if t >= 0.0 and dir3Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            dir3Resp.tStart = t  # underestimates by a little under one frame
            dir3Resp.frameNStart = frameN  # exact frame index
            dir3Resp.status = STARTED
            # AllowedKeys looks like a variable named `left_allowed_keys`
            if not 'left_allowed_keys' in locals():
                logging.error('AllowedKeys variable `left_allowed_keys` is not defined.')
                core.quit()
            if not type(left_allowed_keys) in [list, tuple, np.ndarray]:
                if not isinstance(left_allowed_keys, basestring):
                    logging.error('AllowedKeys variable `left_allowed_keys` is not string- or list-like.')
                    core.quit()
                elif not ',' in left_allowed_keys: left_allowed_keys = (left_allowed_keys,)
                else:  left_allowed_keys = eval(left_allowed_keys)
            # keyboard checking is just starting
            win.callOnFlip(dir3Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if dir3Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=list(left_allowed_keys))
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                dir3Resp.keys = theseKeys[-1]  # just the last key pressed
                dir3Resp.rt = dir3Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        '''if event.getKeys(keyList = all_allowed_keys):
            npresses += 1
            if npresses == maxPresses:
                continueRoutine = False
            if npresses == 1:
                directions3img.image = imPracRight
                directions3Text.text = 'MIDDLE FINGER'
        '''
        '''
        if npresses==0 and event.getKeys(keyList = left_allowed_keys):
            npresses+=1
            directions3img.image = imPracRight
            directions3Text.text = 'MIDDLE FINGER'
        if npresses==1 and event.getKeys(keyList = right_allowed_keys):
            continueRoutine = False
        '''
        #    if npresses == 3:
        #        directions4img.image = imPrac4a
        #        directions4Text.text = 'Press the button under your index finger to indicate that this picture is "NEW".'
        #    if npresses == 4:
        #        directions4img.image = imPrac5a
        #        directions4Text.text = 'Press the button under your index finger to indicate that this picture is "NEW".'
        #    if npresses == 5:
        #        directions4img.image = imPrac2a
        #        directions4Text.text = 'This picture is exactly the same as one that you saw earlier. \n\nSo, for this picture, press the button under your middle finger to indicate that it is "OLD".'
        #    if npresses == 6:
        #        directions4img.image = imPrac1b
        #        directions4Text.text = 'This picture looks similar to one you saw earlier.'
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions3Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions3"-------
    for thisComponent in directions3Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if dir3Resp.keys in ['', [], None]:  # No response was made
       dir3Resp.keys=None
    # store data for runInstructions (TrialHandler)
    runInstructions.addData('dir3Resp.keys',dir3Resp.keys)
    if dir3Resp.keys != None:  # we had a response
        runInstructions.addData('dir3Resp.rt', dir3Resp.rt)
    
    
    # the Routine "directions3" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directions3b"-------
    t = 0
    directions3bClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    dir3bResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    dir3bResp.status = NOT_STARTED
    raVerbalText.text='"When the RED squares are on the RIGHT, press quickly with your MIDDLE finger. Try pressing with your MIDDLE finger now."'
    raText.text='\n\nWait for participant to respond to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    
    # keep track of which components have finished
    directions3bComponents = []
    directions3bComponents.append(directions3bText)
    directions3bComponents.append(directions3bimg)
    directions3bComponents.append(middleFinger)
    directions3bComponents.append(dir3bResp)
    for thisComponent in directions3bComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions3b"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions3bClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *directions3bText* updates
        if t >= 0.0 and directions3bText.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3bText.tStart = t  # underestimates by a little under one frame
            directions3bText.frameNStart = frameN  # exact frame index
            directions3bText.setAutoDraw(True)
        
        # *directions3bimg* updates
        if t >= 0.0 and directions3bimg.status == NOT_STARTED:
            # keep track of start time/frame for later
            directions3bimg.tStart = t  # underestimates by a little under one frame
            directions3bimg.frameNStart = frameN  # exact frame index
            directions3bimg.setAutoDraw(True)
        
        # *middleFinger* updates
        if t >= 0.0 and middleFinger.status == NOT_STARTED:
            # keep track of start time/frame for later
            middleFinger.tStart = t  # underestimates by a little under one frame
            middleFinger.frameNStart = frameN  # exact frame index
            middleFinger.setAutoDraw(True)
        
        # *dir3bResp* updates
        if t >= 0.0 and dir3bResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            dir3bResp.tStart = t  # underestimates by a little under one frame
            dir3bResp.frameNStart = frameN  # exact frame index
            dir3bResp.status = STARTED
            # AllowedKeys looks like a variable named `right_allowed_keys`
            if not 'right_allowed_keys' in locals():
                logging.error('AllowedKeys variable `right_allowed_keys` is not defined.')
                core.quit()
            if not type(right_allowed_keys) in [list, tuple, np.ndarray]:
                if not isinstance(right_allowed_keys, basestring):
                    logging.error('AllowedKeys variable `right_allowed_keys` is not string- or list-like.')
                    core.quit()
                elif not ',' in right_allowed_keys: right_allowed_keys = (right_allowed_keys,)
                else:  right_allowed_keys = eval(right_allowed_keys)
            # keyboard checking is just starting
            win.callOnFlip(dir3bResp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if dir3bResp.status == STARTED:
            theseKeys = event.getKeys(keyList=list(right_allowed_keys))
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                dir3bResp.keys = theseKeys[-1]  # just the last key pressed
                dir3bResp.rt = dir3bResp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions3bComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions3b"-------
    for thisComponent in directions3bComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if dir3bResp.keys in ['', [], None]:  # No response was made
       dir3bResp.keys=None
    # store data for runInstructions (TrialHandler)
    runInstructions.addData('dir3bResp.keys',dir3bResp.keys)
    if dir3bResp.keys != None:  # we had a response
        runInstructions.addData('dir3bResp.rt', dir3bResp.rt)
    
    # the Routine "directions3b" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "runDirections3"-------
    t = 0
    runDirections3Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    runDirections3Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    runDirections3Resp.status = NOT_STARTED
    raVerbalText.text='"Do you have any questions? Are you ready to play the game and stay still in this scan?"'
    raText.text='\n\nPress <space> to continue.'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # keep track of which components have finished
    runDirections3Components = []
    runDirections3Components.append(runDirections3Text)
    runDirections3Components.append(runDirections3Resp)
    for thisComponent in runDirections3Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "runDirections3"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = runDirections3Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *runDirections3Text* updates
        if t >= 0.0 and runDirections3Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            runDirections3Text.tStart = t  # underestimates by a little under one frame
            runDirections3Text.frameNStart = frameN  # exact frame index
            runDirections3Text.setAutoDraw(True)
        
        # *runDirections3Resp* updates
        if t >= 0.0 and runDirections3Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            runDirections3Resp.tStart = t  # underestimates by a little under one frame
            runDirections3Resp.frameNStart = frameN  # exact frame index
            runDirections3Resp.status = STARTED
            # keyboard checking is just starting
            event.clearEvents(eventType='keyboard')
        if runDirections3Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in runDirections3Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "runDirections3"-------
    for thisComponent in runDirections3Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    raVerbalText.text=''
    raText.text=''
    raVerbalText.draw()
    raText.draw()
    raWin.flip()
    # the Routine "runDirections3" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed mode != 'practice' repeats of 'runInstructions'


# set up handler to look after randomisation of conditions etc
runs = data.TrialHandler(nReps=1, method='sequential', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='runs')
thisExp.addLoop(runs)  # add the loop to the experiment
thisRun = runs.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisRun.rgb)
if thisRun != None:
    for paramName in thisRun.keys():
        exec(paramName + '= thisRun.' + paramName)

for thisRun in runs:
    currentLoop = runs
    # abbreviate parameter names if possible (e.g. rgb = thisRun.rgb)
    if thisRun != None:
        for paramName in thisRun.keys():
            exec(paramName + '= thisRun.' + paramName)
    
    #------Prepare to start Routine "waitForScanner"-------
    t = 0
    waitForScannerClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    runNum = runNumber + runs.thisN
    raText.pos = [0, 0]
    if mode != 'practice':
        raText.text='Start the scan.'
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
    # keep track of which components have finished
    waitForScannerComponents = []
    waitForScannerComponents.append(waitForScannerText)
    for thisComponent in waitForScannerComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "waitForScanner"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = waitForScannerClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *waitForScannerText* updates
        if t >= 0.0 and waitForScannerText.status == NOT_STARTED:
            # keep track of start time/frame for later
            waitForScannerText.tStart = t  # underestimates by a little under one frame
            waitForScannerText.frameNStart = frameN  # exact frame index
            waitForScannerText.setAutoDraw(True)
        if expInfo['mriMode'] != 'Off':
            winRecorder.keyframe()
            
            if trigger == 'usb':
                vol = launchScan(win, MR_settings, 
                      globalClock=fmriClock,
                      wait_msg='',
                      mode=expInfo['mriMode'])
        
            elif trigger == 'parallel':
                address = 0x378
                #parallel.setPortAddress(0x378)
                wait_msg = "Get Ready!"
                pinStatus = winioport.inp(address)
                waitMsgStim = visual.TextStim(win, color='white', text=wait_msg)
                waitMsgStim.draw()
                win.flip()
                while True:
                    if pinStatus != winioport.inp(address):
                       break
                       # start exp when pin values change
                fmriClock.reset()
                logging.exp('parallel trigger: start of scan')
                win.flip()  # blank the screen on first sync pulse received
        else:
            fmriClock.reset()
        
        triggerWallTime = datetime.datetime.today()
        expInfo['triggerWallTime'] = triggerWallTime.strftime('%a %b %d %H:%M:%S %Y')
        expInfo['triggerGlobalTime'] = globalClock.getTime()
        continueRoutine = False
        
        filename = _thisDir + os.sep + u'data/' + '_'.join([expName, expInfo['sessionID'], 'run%s'%runNum, triggerWallTime.strftime('%Y-%m-%d_%H%M%S')])
        continueRoutine = False
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in waitForScannerComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "waitForScanner"-------
    for thisComponent in waitForScannerComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    raText.text=''
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # the Routine "waitForScanner" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    # set up handler to look after randomisation of conditions etc
    countdowns = data.TrialHandler(nReps=4, method='random', 
        extraInfo=expInfo, originPath=-1,
        trialList=[None],
        seed=None, name='countdowns')
    thisExp.addLoop(countdowns)  # add the loop to the experiment
    thisCountdown = countdowns.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisCountdown.rgb)
    if thisCountdown != None:
        for paramName in thisCountdown.keys():
            exec(paramName + '= thisCountdown.' + paramName)
    
    for thisCountdown in countdowns:
        currentLoop = countdowns
        # abbreviate parameter names if possible (e.g. rgb = thisCountdown.rgb)
        if thisCountdown != None:
            for paramName in thisCountdown.keys():
                exec(paramName + '= thisCountdown.' + paramName)
        
        #------Prepare to start Routine "countdown"-------
        t = 0
        countdownClock.reset()  # clock 
        frameN = -1
        routineTimer.add(2.000000)
        # update component parameters for each repeat
        countdownTime = fmriClock.getTime()
        countdownMsg = currentLoop.nRemaining + 1
        
        countdownText.setText(countdownMsg)
        record_expt_win = True
        
        noResp=0
        
        leftCount = 0
        rightCount=0
        
        leftCorrCount=0
        rightCorrCount=0
        
        totalCorrCount=0
        
        leftAcc=0
        rightAcc=0
        totalAcc=0
        
        totalAccList=[]
        totalFiveBackAvg = 0
        
        totalButtonTrack = []
        countConsecutive=1
        # keep track of which components have finished
        countdownComponents = []
        countdownComponents.append(countdownText)
        for thisComponent in countdownComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "countdown"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = countdownClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            # After first frame is drawn
            if frameN == 1:
                if 'runs.thisN' in thisExp.entries:
                    updateRunLog(thisExp.entries)
            
            # *countdownText* updates
            if t >= 0 and countdownText.status == NOT_STARTED:
                # keep track of start time/frame for later
                countdownText.tStart = t  # underestimates by a little under one frame
                countdownText.frameNStart = frameN  # exact frame index
                countdownText.setAutoDraw(True)
            if countdownText.status == STARTED and t >= (0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                countdownText.setAutoDraw(False)
            if record_expt_win:
                winRecorder.keyframe()
                record_expt_win = False
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in countdownComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "countdown"-------
        for thisComponent in countdownComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        currentLoop.addData('countdownStartTime', countdownTime)
        currentLoop.addData('countdownLabel', countdownMsg)
        currentLoop.addData('trialNum', 0)
        
        
        thisExp.nextEntry()
        
    # completed 4 repeats of 'countdowns'
    
    
    # set up handler to look after randomisation of conditions etc
    blocks = data.TrialHandler(nReps=1, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=data.importConditions(blocksList),
        seed=None, name='blocks')
    thisExp.addLoop(blocks)  # add the loop to the experiment
    thisBlock = blocks.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisBlock.rgb)
    if thisBlock != None:
        for paramName in thisBlock.keys():
            exec(paramName + '= thisBlock.' + paramName)
    
    for thisBlock in blocks:
        currentLoop = blocks
        # abbreviate parameter names if possible (e.g. rgb = thisBlock.rgb)
        if thisBlock != None:
            for paramName in thisBlock.keys():
                exec(paramName + '= thisBlock.' + paramName)
        
        #------Prepare to start Routine "fixRoutine"-------
        t = 0
        fixRoutineClock.reset()  # clock 
        frameN = -1
        routineTimer.add(18.000000)
        # update component parameters for each repeat
        
        fixRoutineText.setColor('green', colorSpace='rgb')
        record_expt_win = True
        # keep track of which components have finished
        fixRoutineComponents = []
        fixRoutineComponents.append(fixRoutineText)
        for thisComponent in fixRoutineComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "fixRoutine"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = fixRoutineClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            if fixRoutineText.color == 'green':
                if t > 17 or (t > practiceFixDur-1 and mode == 'practice'):
                    fixRoutineText.color = 'white'
                    record_expt_win = True
            if t > practiceFixDur and mode == 'practice':
                continueRoutine = False
            if frameN == 0:
                fixStartTime = fmriClock.getTime()
                blocks.addData('fixStartTime', fixStartTime)
                events.append(dict(
                    run=runNum,
                    onset=fixStartTime,
                    duration=18
                ))
            
            # *fixRoutineText* updates
            if t >= 0.0 and fixRoutineText.status == NOT_STARTED:
                # keep track of start time/frame for later
                fixRoutineText.tStart = t  # underestimates by a little under one frame
                fixRoutineText.frameNStart = frameN  # exact frame index
                fixRoutineText.setAutoDraw(True)
            if fixRoutineText.status == STARTED and t >= (0.0 + (18-win.monitorFramePeriod*0.75)): #most of one frame period left
                fixRoutineText.setAutoDraw(False)
            if record_expt_win:
                winRecorder.keyframe()
                record_expt_win = False
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in fixRoutineComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "fixRoutine"-------
        for thisComponent in fixRoutineComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        blocks.addData('fixEndTime', fmriClock.getTime())
        
        # In the practice this routine stops early (not non-slip safe) so restart manually
        routineTimer.reset()
        
        
        # set up handler to look after randomisation of conditions etc
        trials = data.TrialHandler(nReps=1, method='sequential', 
            extraInfo=expInfo, originPath=-1,
            trialList=data.importConditions(trialsList),
            seed=None, name='trials')
        thisExp.addLoop(trials)  # add the loop to the experiment
        thisTrial = trials.trialList[0]  # so we can initialise stimuli with some values
        # abbreviate parameter names if possible (e.g. rgb=thisTrial.rgb)
        if thisTrial != None:
            for paramName in thisTrial.keys():
                exec(paramName + '= thisTrial.' + paramName)
        
        for thisTrial in trials:
            currentLoop = trials
            # abbreviate parameter names if possible (e.g. rgb = thisTrial.rgb)
            if thisTrial != None:
                for paramName in thisTrial.keys():
                    exec(paramName + '= thisTrial.' + paramName)
            
            #------Prepare to start Routine "trial"-------
            t = 0
            trialClock.reset()  # clock 
            frameN = -1
            routineTimer.add(3.000000)
            # update component parameters for each repeat
            if side == 'right':
                angularOffset = 5  # 7
            else:  # side == 'left'
                angularOffset = 21  # 27
            
            radialOut = 6
            
            targetOri = baseOriOffset + ((wedgeLength / 2) + wedgeLength * angularOffset)
            radialMask = makeMask([radialOut, radialOut+1], radialCycles * 2)
            target1.mask = radialMask
            target1.ori = targetOri
            
            ##angularOffset = 18
            targetOri = baseOriOffset + ((wedgeLength / 2) + wedgeLength * (angularOffset + 2))
            radialMask = makeMask([radialOut+2, radialOut+3], radialCycles * 2)
            target2.mask = radialMask
            target2.ori = targetOri
            
            target1.status = NOT_STARTED
            target2.status = NOT_STARTED
            
            
            #win.recordFrameIntervals = True
            
            # Number of frames to wait before starting target (ISI)
            # minus 3 frames worth of slop (current time to write checkerboard and start trial)
            fixFrames = int(framesPerSecond * (fixDur-.5))
            logging.exp('Fixation Frame count: %d' % fixFrames)
            
            #trialCountdown.add(fixDur + targetDur)  # 3 s duration
            earlyISI = event.BuilderKeyResponse()  # create an object of type KeyResponse
            earlyISI.status = NOT_STARTED
            trialResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
            trialResp.status = NOT_STARTED
            lateISI = event.BuilderKeyResponse()  # create an object of type KeyResponse
            lateISI.status = NOT_STARTED
            number_trials = ((trials.thisN + 1) + (blocks.thisN*9))
            currentLoop.addData('trialNum', number_trials)
            if mode == 'practice':
                raText.text = 'Trial # %s of 9 in this scan.\n\nTotal accuracy: %04.2f %% \nLeft accuracy: %04.2f %% \nRight accuracy: %04.2f %% \n\nTotal accuracy last 5 trials: %04.2f %% \n\nTotal no responses: %s \n\n# same consecutive (no) responses: %s' % (number_trials,totalAcc*100,leftAcc*100,rightAcc*100,totalFiveBackAvg*100,noResp,countConsecutive)
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            else:
                raText.text = 'Trial # %s of 27 in this scan.\n\nTotal accuracy: %04.2f %% \nLeft accuracy: %04.2f %% \nRight accuracy: %04.2f %% \n\nTotal accuracy last 5 trials: %04.2f %% \n\nTotal no responses: %s \n\n# same consecutive (no) responses: %s' % (number_trials,totalAcc*100,leftAcc*100,rightAcc*100,totalFiveBackAvg*100,noResp,countConsecutive)
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            
            record_expt_win = True
            # keep track of which components have finished
            trialComponents = []
            trialComponents.append(earlyISI)
            trialComponents.append(trialResp)
            trialComponents.append(lateISI)
            trialComponents.append(centerCircle)
            trialComponents.append(fixText)
            for thisComponent in trialComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "trial"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = trialClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                for prompt in buttonPrompts:
                    prompt.draw()
                
                flicker_frames += 1
                if flicker_frames >= flickerFramesPerHalfCycle:
                    checkerboard.contrast = checkerboard.contrast * -1
                    flicker_frames = 0
                    record_expt_win = True
                
                # Draw the background first
                checkerboard.draw()
                
                if frameN > fixFrames and target1.status == NOT_STARTED:
                    target1.autoDraw = True
                    target1.status = STARTED
                
                    target2.autoDraw = True
                    target2.status = STARTED
                    
                    if currentLoop.name == 'trials':
                        targetStartTime = fmriClock.getTime()
                        trials.addData('targetStartTime', targetStartTime)
                        events.append(dict(
                            run=runNum,
                            onset=checkerboardStartTime,
                            condition='target',
                            duration=.5))
                
                if frameN > (fixFrames + targetFrames):
                
                    if target1.status == STARTED:
                        target1.autoDraw = False
                        target2.autoDraw = False
                        target1.status == FINISHED
                        target2.status == FINISHED
                        currentLoop.addData('targetEndTime', fmriClock.getTime())
                
                if frameN == 0 and currentLoop.name == 'trials':
                    checkerboardStartTime = fmriClock.getTime()
                    trials.addData('checkerboardStartTime', checkerboardStartTime)
                    trialComponents.extend([target1, target2])
                
                
                # *earlyISI* updates
                if t >= 0.0 and earlyISI.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    earlyISI.tStart = t  # underestimates by a little under one frame
                    earlyISI.frameNStart = frameN  # exact frame index
                    earlyISI.status = STARTED
                    # AllowedKeys looks like a variable named `allowed_keys`
                    if not 'allowed_keys' in locals():
                        logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                        core.quit()
                    if not type(allowed_keys) in [list, tuple, np.ndarray]:
                        if not isinstance(allowed_keys, basestring):
                            logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                            core.quit()
                        elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                        else:  allowed_keys = eval(allowed_keys)
                    # keyboard checking is just starting
                    win.callOnFlip(earlyISI.clock.reset)  # t=0 on next screen flip
                    event.clearEvents(eventType='keyboard')
                if earlyISI.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                    earlyISI.status = STOPPED
                if earlyISI.status == STARTED:
                    theseKeys = event.getKeys(keyList=list(allowed_keys))
                    
                    # check for quit:
                    if "escape" in theseKeys:
                        endExpNow = True
                    if len(theseKeys) > 0:  # at least one key was pressed
                        earlyISI.keys.extend(theseKeys)  # storing all keys
                        earlyISI.rt.append(earlyISI.clock.getTime())
                
                # *trialResp* updates
                if t >= 2 and trialResp.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    trialResp.tStart = t  # underestimates by a little under one frame
                    trialResp.frameNStart = frameN  # exact frame index
                    trialResp.status = STARTED
                    # AllowedKeys looks like a variable named `allowed_keys`
                    if not 'allowed_keys' in locals():
                        logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                        core.quit()
                    if not type(allowed_keys) in [list, tuple, np.ndarray]:
                        if not isinstance(allowed_keys, basestring):
                            logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                            core.quit()
                        elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                        else:  allowed_keys = eval(allowed_keys)
                    # keyboard checking is just starting
                    win.callOnFlip(trialResp.clock.reset)  # t=0 on next screen flip
                    event.clearEvents(eventType='keyboard')
                if trialResp.status == STARTED and t >= (2 + (.7-win.monitorFramePeriod*0.75)): #most of one frame period left
                    trialResp.status = STOPPED
                if trialResp.status == STARTED:
                    theseKeys = event.getKeys(keyList=list(allowed_keys))
                    
                    # check for quit:
                    if "escape" in theseKeys:
                        endExpNow = True
                    if len(theseKeys) > 0:  # at least one key was pressed
                        trialResp.keys.extend(theseKeys)  # storing all keys
                        trialResp.rt.append(trialResp.clock.getTime())
                
                # *lateISI* updates
                if t >= 2.7 and lateISI.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    lateISI.tStart = t  # underestimates by a little under one frame
                    lateISI.frameNStart = frameN  # exact frame index
                    lateISI.status = STARTED
                    # AllowedKeys looks like a variable named `allowed_keys`
                    if not 'allowed_keys' in locals():
                        logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                        core.quit()
                    if not type(allowed_keys) in [list, tuple, np.ndarray]:
                        if not isinstance(allowed_keys, basestring):
                            logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                            core.quit()
                        elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                        else:  allowed_keys = eval(allowed_keys)
                    # keyboard checking is just starting
                    win.callOnFlip(lateISI.clock.reset)  # t=0 on next screen flip
                    event.clearEvents(eventType='keyboard')
                if lateISI.status == STARTED and t >= (2.7 + (.3-win.monitorFramePeriod*0.75)): #most of one frame period left
                    lateISI.status = STOPPED
                if lateISI.status == STARTED:
                    theseKeys = event.getKeys(keyList=list(allowed_keys))
                    
                    # check for quit:
                    if "escape" in theseKeys:
                        endExpNow = True
                    if len(theseKeys) > 0:  # at least one key was pressed
                        lateISI.keys.extend(theseKeys)  # storing all keys
                        lateISI.rt.append(lateISI.clock.getTime())
                
                # *centerCircle* updates
                if t >= 0 and centerCircle.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    centerCircle.tStart = t  # underestimates by a little under one frame
                    centerCircle.frameNStart = frameN  # exact frame index
                    centerCircle.setAutoDraw(True)
                if centerCircle.status == STARTED and t >= (0 + (3-win.monitorFramePeriod*0.75)): #most of one frame period left
                    centerCircle.setAutoDraw(False)
                
                # *fixText* updates
                if t >= 0.0 and fixText.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    fixText.tStart = t  # underestimates by a little under one frame
                    fixText.frameNStart = frameN  # exact frame index
                    fixText.setAutoDraw(True)
                if fixText.status == STARTED and t >= (0.0 + (3-win.monitorFramePeriod*0.75)): #most of one frame period left
                    fixText.setAutoDraw(False)
                if frameN == 0 and trials.thisN == 0:
                    blockStartTime = fmriClock.getTime()
                    currentLoop.addData('blockStartTime', blockStartTime)
                if frameN == 1:
                    updateRunLog(thisExp.entries)
                
                
                
                if record_expt_win:
                    winRecorder.keyframe()
                    record_expt_win = False
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in trialComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "trial"-------
            for thisComponent in trialComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            updateRunLog(thisExp.entries)
            
            # check responses
            if earlyISI.keys in ['', [], None]:  # No response was made
               earlyISI.keys=None
            # store data for trials (TrialHandler)
            trials.addData('earlyISI.keys',earlyISI.keys)
            if earlyISI.keys != None:  # we had a response
                trials.addData('earlyISI.rt', earlyISI.rt)
            # check responses
            if trialResp.keys in ['', [], None]:  # No response was made
               trialResp.keys=None
            # store data for trials (TrialHandler)
            trials.addData('trialResp.keys',trialResp.keys)
            if trialResp.keys != None:  # we had a response
                trials.addData('trialResp.rt', trialResp.rt)
            # check responses
            if lateISI.keys in ['', [], None]:  # No response was made
               lateISI.keys=None
            # store data for trials (TrialHandler)
            trials.addData('lateISI.keys',lateISI.keys)
            if lateISI.keys != None:  # we had a response
                trials.addData('lateISI.rt', lateISI.rt)
            if trialResp.keys:
                firstKey = trialResp.keys[0]
                firstRt = trialResp.rt[0]
                correct = firstKey in config['keys'][side]
            else:
                firstKey = None
                firstRt = None
                correct = False
            
            currentLoop.addData('correct', correct)
            currentLoop.addData('firstKey', firstKey)
            currentLoop.addData('firstRt', firstRt)
            
            resp=None
            respLast=None
            
            if trialResp.keys:
                if (firstKey in config['keys']['left'] and side == 'left'): #correct left
                    resp='left'
                    feedbackMsg = 'CORRECT'
                    correct_response = 1
                    leftCount+=1
                    leftCorrCount += 1
                    totalCorrCount += 1
                    leftAcc= (leftCorrCount)/(leftCount)
                elif (firstKey in config['keys']['right'] and side == 'right'): #correct right
                    resp='right'
                    feedbackMsg = 'CORRECT'
                    correct_response = 1
                    rightCount+=1
                    rightCorrCount += 1
                    totalCorrCount += 1
                    rightAcc= (rightCorrCount)/(rightCount)
                elif (firstKey in config['keys']['right'] and side == 'left'): #pressed right to left
                    resp='right'
                    feedbackMsg = 'INCORRECT'
                    correct_response = 0
                    leftCount+=1
                    leftAcc= (leftCorrCount)/(leftCount)
                elif (firstKey in config['keys']['left'] and side == 'right'): #pressed left to right
                    resp='left'
                    feedbackMsg = 'INCORRECT'
                    correct_response = 0
                    rightCount+=1
                    rightAcc= (rightCorrCount)/(rightCount)
            elif (not trialResp.keys and side == 'left'): #no response
                    feedbackMsg = 'TOO SLOW'
                    correct_response = 0
                    noResp+=1
                    leftCount+=1
                    leftAcc= (leftCorrCount)/(leftCount)
            elif (not trialResp.keys and side == 'right'): #no response
                    feedbackMsg = 'TOO SLOW'
                    correct_response = 0
                    noResp+=1
                    rightCount+=1
                    rightAcc= (rightCorrCount)/(rightCount)
            
            totalAccList.append(correct_response)
            totalAcc=(totalCorrCount)/(number_trials)
            
            #coding last button press made
            if trialResp.keys:
                if trialResp.keys[-1] in config['keys']['left']:
                    respLast='left'
                elif trialResp.keys[-1] in config['keys']['right']:
                    respLast='right'
            
            #TotalAcc5back
            if number_trials >= 5:
                totalFiveBackAvg = 0
                for i in range(-1, -6,-1):
                    totalFiveBackAvg+=totalAccList[i]
                totalFiveBackAvg=totalFiveBackAvg/5.0
            
            #track # consecutive responses with same button:
            if resp==str('left'):
                totalButtonTrack.append(1)
            elif resp==str('right'):
                totalButtonTrack.append(2)
            else: #no response
                totalButtonTrack.append(0)
            
            if number_trials >=2:
                if totalButtonTrack[-1]==totalButtonTrack[-2]:
                    countConsecutive+=1
                elif totalButtonTrack[-1]!=totalButtonTrack[-2]:
                    countConsecutive=1
            
            currentLoop.addData('feedback', feedbackMsg)
            
            currentLoop.addData('resp', resp)
            currentLoop.addData('respLast', respLast)
            currentLoop.addData('noResp', noResp)
            
            currentLoop.addData('correctResponse', correct_response)
            
            currentLoop.addData('leftCount', leftCount)
            currentLoop.addData('rightCount', rightCount)
            
            currentLoop.addData('leftCorrCount', leftCorrCount)
            currentLoop.addData('rightCorrCount', rightCorrCount)
            
            currentLoop.addData('leftAcc', leftAcc)
            currentLoop.addData('rightAcc', rightAcc)
            currentLoop.addData('totalAcc', totalAcc)
            
            #currentLoop.addData('totalAccList', totalAccList)
            currentLoop.addData('totalFiveBackAvg', totalFiveBackAvg)
            
            #currentLoop.addData('totalButtonTrack', totalButtonTrack)
            currentLoop.addData('countConsecutive', countConsecutive)
            
            if (currentLoop.name == 'trials' and trials.nRemaining == 0):
                currentLoop.addData('blockEndTime', fmriClock.getTime())
            
            
            
            thisExp.nextEntry()
            
        # completed 1 repeats of 'trials'
        
        thisExp.nextEntry()
        
    # completed 1 repeats of 'blocks'
    
    
    #------Prepare to start Routine "endFixation"-------
    t = 0
    endFixationClock.reset()  # clock 
    frameN = -1
    routineTimer.add(12.200000)
    # update component parameters for each repeat
    
    # keep track of which components have finished
    endFixationComponents = []
    endFixationComponents.append(endFixationText)
    for thisComponent in endFixationComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "endFixation"-------
    continueRoutine = True
    while continueRoutine and routineTimer.getTime() > 0:
        # get current time
        t = endFixationClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *endFixationText* updates
        if t >= 0.0 and endFixationText.status == NOT_STARTED:
            # keep track of start time/frame for later
            endFixationText.tStart = t  # underestimates by a little under one frame
            endFixationText.frameNStart = frameN  # exact frame index
            endFixationText.setAutoDraw(True)
        if endFixationText.status == STARTED and t >= (0.0 + (12.2-win.monitorFramePeriod*0.75)): #most of one frame period left
            endFixationText.setAutoDraw(False)
        if frameN == 0:
            fixStartTime = fmriClock.getTime()
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in endFixationComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "endFixation"-------
    for thisComponent in endFixationComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    fixEndTime = runEndTime = fmriClock.getTime()
    
    # Add last fix and end time. Go two back to get the last trial
    # after thisExp.nextEntry has been called at the end of trials
    # and blocks.
    entries = thisExp.entries
    entries[-2]['fixStartTime'] = fixStartTime
    entries[-2]['fixEndTime'] = fixEndTime
    entries[-2]['runEndTime'] = runEndTime
    updateRunLog(thisExp.entries)
    
    #------Prepare to start Routine "endRun"-------
    t = 0
    endRunClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    
    endRunResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    endRunResp.status = NOT_STARTED
    raText.text='Press <space> to exit.'
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # keep track of which components have finished
    endRunComponents = []
    endRunComponents.append(endRunText)
    endRunComponents.append(endRunResp)
    for thisComponent in endRunComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "endRun"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = endRunClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        
        # *endRunText* updates
        if t >= 0.0 and endRunText.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunText.tStart = t  # underestimates by a little under one frame
            endRunText.frameNStart = frameN  # exact frame index
            endRunText.setAutoDraw(True)
        
        # *endRunResp* updates
        if t >= 0.0 and endRunResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunResp.tStart = t  # underestimates by a little under one frame
            endRunResp.frameNStart = frameN  # exact frame index
            endRunResp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(endRunResp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if endRunResp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                endRunResp.keys = theseKeys[-1]  # just the last key pressed
                endRunResp.rt = endRunResp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in endRunComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "endRun"-------
    for thisComponent in endRunComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # check responses
    if endRunResp.keys in ['', [], None]:  # No response was made
       endRunResp.keys=None
    # store data for runs (TrialHandler)
    runs.addData('endRunResp.keys',endRunResp.keys)
    if endRunResp.keys != None:  # we had a response
        runs.addData('endRunResp.rt', endRunResp.rt)
    
    # the Routine "endRun" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed 1 repeats of 'runs'






save_movie(filename, winRecorder, raWinRecorder)


































# these shouldn't be strictly necessary (should auto-save)
thisExp.saveAsPickle(filename)
logging.flush()
# make sure everything is closed down
thisExp.abort() # or data files will save again on exit
win.close()
core.quit()
